/*
 * Store number of picks for each station and compute average picks per station per hour
 */
package pickstatistics;

import cvarg.seismic.earthworm.messages.PickSCNL;
import java.util.Vector;

public class PickCounter {

    public Vector<PickSCNL> picks = new Vector<PickSCNL>();
    public String Station;
    public String Channel;

    /*
     * Private construct for new station
     */
    private PickCounter(PickSCNL firstPick) {
        this.Station = firstPick.Station;
        this.Channel = firstPick.Channel;
    }

    /*
     * Add a pick to this station
     */
    public void addPick(PickSCNL pick) {
        picks.add(pick);
    }

    /*
     * Receives a new pick and checks if the station is already considered.
     * If so, adds the pick to that station. If not, creates a new station
     * and adds the pick as its first pick.
     */
    public static void addPicker(PickSCNL firstPick, Vector<PickCounter> pickers) {
        PickCounter newPicker = new PickCounter(firstPick);
        for (int i = 0; i < pickers.size(); i++) {
            if ((pickers.elementAt(i).Station.equals(firstPick.Station))
                    && (pickers.elementAt(i).Channel.equals(firstPick.Channel))) {
                pickers.elementAt(i).addPick(firstPick);
                return;
            }
        }
        pickers.add(newPicker);
    }

    /* Methods to address pick statistics */

    /*
     * Return number of picks
     */
    public int pickCount() {
        return picks.size();
    }
    /*
     * Computes the evolution of picks taking into account an integration time
     * integration: in seconds
     */
    public int[] pickEvolution(int integration) {
        long[] interval = getTemporalInterval(); //Gets the maximum time interval for all the picks
        long delta = interval[1]-interval[0];
        /* Check if integration time is larger than the available interval */
        if (delta<integration) {
            //Count all picks
            return new int[]{picks.size()};
        }
        /* Integrate number of picks over the considered time interval */
        int[] output = new int[(int)(delta / integration) + 1];
        for (int i=0; i<picks.size(); i++) {
            long deltaTime = (long)(picks.elementAt(i).arrivalTime * 1000) - interval[0];

            int pos = (int)(deltaTime / integration);
            //System.out.println(deltaTime + " " + pos);
            output[pos]++;
        }
        return output;
    }

    /* Compute general average picks in the integration time */
    public double pickAverage(int integration) {
        int[] pickEvo = pickEvolution(integration);
        double output = 0;
        for (int i=0; i<pickEvo.length; i++) {
            output += (double) pickEvo[i];
        }
        output /= pickEvo.length;
        return output;
    }


    /* Utilities for pick counting */
    private long[] getTemporalInterval() {
        if (picks.size()==0) {
            return new long[]{0, 0};
        }
        long minTime = Long.MAX_VALUE;
        long maxTime = 0;
        for (int i=0; i<picks.size(); i++) {
            if ((long)(picks.elementAt(i).arrivalTime*1000)<minTime) {
                minTime = (long)(picks.elementAt(i).arrivalTime * 1000);
            }
            if ((long)(picks.elementAt(i).arrivalTime*1000)>maxTime) {
                maxTime = (long)(picks.elementAt(i).arrivalTime * 1000);
            }
        }
        return new long[]{minTime, maxTime};
    }
}
